#include "lbs_i.h"

Define_Module( LBS_I )

void LBS_I::activity()
{
    // get parameters
    simtime_t delta_t = par("delta_t");

    // search servers and save reference to its workload parameter
    // as well as their rate
    cModule *compound = parentModule();

    for (int i = 0; i<serverCount; i++) {
        cModule *svr = compound->submodule("server", i);
        if ((svr==0) || (strcmp(svr->className(), "Server")!=0)) {
            throw cException("server[] contains non-Server");
        }
        server[i] = (Server*) svr;
        rate[i] = svr->par("rate");
    }

    // create update-message and issue it immediately
    cMessage *updateMsg = new cMessage("update-balancer");
    scheduleAt(simTime() + delta_t, updateMsg);
    update();

    // message loop
    for (;;) {
        // get message
        cMessage *msg = receive();

        if (msg==updateMsg) {
            // update received, so do it and issue the message again delta_t secs later
            update();
            scheduleAt(simTime() + delta_t, updateMsg);
        } else {
            // send the message to the current server to use
            send(msg, "out", serverToUse);
        }
    }

}

void LBS_I::update()
{
    // search server with minimum workload (count the number of occurences)
    int i;
    int count = 0;
    double load = 100000000;
    for (i = 0; i<serverCount; i++) {
        // get queue length
        long l = server[i]->getJobsInSystem();
        if (l>0) --l;
        // calculate performance measure and save it
        double bl = (double) l/rate[i];
        balancing[i] = bl;
        if (bl<load) {
            load = bl;
            count = 1;
        } else if (bl==load) {
            count++;
        }
    }
    // choose one server with minimum workload randomly
    int serverNo = (int) uniform(0, count, rg);
    // find this server
    for (i = 0; i<serverCount; i++) {
        if (load==balancing[i]) {
            if (serverNo==0) break;
            serverNo--;
        }
    }
    // check on error
    if (serverNo!=0) throw cException("error while choosing server");

    // use the selected server until next update()
    serverToUse = i;
}

void LBS_I::initialize()
{
    // get parameters
    rg = par("random_generator");
    serverCount = par("server_count");
    // create arrays
    server = new Server *[serverCount];
    rate = new double[serverCount];
    balancing = new double[serverCount];
}

void LBS_I::finish()
{
    delete [] server;
    delete [] rate;
    delete [] balancing;
}
